==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/dochazka/monitorovanicontroller.php <==
<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;


use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\Http\HttpResponse;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\System\FotoOutputObject;
use RonPortal\User;

class MonitorovaniController extends HttpController
{
    const SETTINGS_POVOLIT_BEZ_PRIHLASENI = 'monitorovanibezprihlaseni';

    /** @var Database */
    protected $database;

    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this ->database = $database;
        $this ->enableAccessWithoutLogin = RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_BEZ_PRIHLASENI);
    }


    /**
     * @param $idPP
     * @return FotoOutputObject
     * @throws Exception
     */
    protected function downloadFoto($idPP)
    {
        // pozor! kontrola prava se neprovadi, pokud je zapnuto monitorovani bez prihlaseni!
        if ( !RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_BEZ_PRIHLASENI) ) {
            Prava::nastavObdobiProPrava(time(), time());

            if (Prava::existujePravo('monitorovani_fotoimg')) {
                $prava = Prava::vratPrava('monitorovani_fotoimg');
            } else if (Prava::existujePravo('monitorovani_foto')) {
                $prava = Prava::vratPrava('monitorovani_foto');
            } else {
                $prava = Prava::vratPrava('monitorovani');
            }

            if ( !$prava->lzeZobrazitPP($idPP, time(), time()) ){
                throw new NeniPravoException;
            }
        }

        $result = new FotoOutputObject($this ->database);
        $result ->nactiIdFotkyPracovnihoPomeru($idPP);

        return $result;
    }

    /**
     * @param $nastaveniOperace
     * @return Redirect
     * @throws Exception
     */
    private function handleVlozeniAbsence($nastaveniOperace)
    {
        if (!is_null($nastaveniOperace)) {
            $osoba = (int)$_POST['osoba'];
            $kodOperace = $nastaveniOperace['operace'];
            $delka = $nastaveniOperace['delka'];

            if ($nastaveniOperace['delka'] > 2) {
                return new Redirect(null,
                    "?menuid=planovani&osoba=$osoba&datum=" . StringUtils::format('date', DateUtils::today()) .
                    "&delka=$delka&backaction=monitorovani&operace=" . urlencode($kodOperace));
            }

            $pPomer = PracovniPomer::nactiPP($this ->database, $osoba);

            $operaceDochazky = OperaceDochazky::nactiOperaci($this->database, $kodOperace);

            $absenceRec = new AbsenceDBRecord($this->database, $pPomer, 'monitorovani');
            $absenceRec->setLogDescription($this);

            $absenceRec->fOperace->setNewValue($operaceDochazky->kod);
            $absenceRec->popisOperace = $operaceDochazky->popis;

            $absenceRec->fDatumOd->setNewValue(DateUtils::today());
            $absenceRec->fDatumDo->setNewValue(DateUtils::today());

            $absenceRec->fDelka->setNewValue($delka);

            if ($nastaveniOperace['typprava']) {
                $prava = Prava::vratPrava($nastaveniOperace['typprava']);
            }
            else {
                $prava = Prava::vratPrava('absence', 'absence_' . $operaceDochazky->kod);
            }

            Prava::nastavObdobiProPrava(DateUtils::today());

            if (!$prava ->lzeZobrazitPP($pPomer ->idPP)) {
                throw new NeniPravoException();
            }

            $absenceRec ->fHistorie ->setNewValue(
                date('j.n.Y G:i') . ', ' . RonPortalUtils::getLogUserName(false) . ': ' .
                TextManager::_('Log_naplanovani_absence') . ' ' .
                date('j.n.Y', $absenceRec ->fDatumOd ->value) . ' - ' . date('j.n.Y', $absenceRec->fDatumDo->value) . "\n");

            if ($operaceDochazky ->autoSchvalovat) {
                $absenceRec->fSchvaleni->setNewValue(1);
            } else {
                $prava = Prava::vratPrava('absence_schval', 'absence_schval_' . $absenceRec ->fOperace ->value);
                if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)  &&  $absenceRec->pp ->lzeZobrazitPP($prava)) {
                    // muze i schvalovat
                    // nastavi schvaleni (f_updatelog = true -> provede pouze v pripade zmeny neceho jineho)
                    $absenceRec ->fSchvaleni ->setNewValue($_POST['Schvaleni'] ? 1 : 0);
                    $absenceRec ->fSchvalil ->setNewValue($_POST['Schvaleni'] ? RonPortalUtils::getLogUserName() : null);
                    $absenceRec ->fDatumSchvaleni ->setNewValue($_POST['Schvaleni'] ? time() : null);
                } else {
                    $absenceRec ->fSchvaleni ->setNewValue(0);
                }
            }

            AbsenceDBRecord::zkontrolujPovoleneDelkyOperace($this->database, $operaceDochazky->kod, $delka);

            AbsenceDBRecord::zkontrolujObdobiAbsence($this ->database, $absenceRec->fDatumOd->value, $absenceRec->fDatumDo->value, $absenceRec->pp);

            AbsenceDBRecord::zkontrolujPrekryvaniAbsenci($this ->database, $absenceRec->pp, $absenceRec->fOperace->value,
                $absenceRec->fDatumOd->value, $absenceRec->fDatumDo->value, $absenceRec->fDelka->value, $absenceRec->fCasOd->value, $absenceRec->fCasDo->value);

            if (RonPortalSettings::getIntValue('zakazatabsencepoukonceni')) {
                if ($absenceRec->pp->datumUkonceni  &&  $absenceRec->fDatumDo->value > $absenceRec->pp->datumUkonceni) {
                    throw new Exception( sprintf(TextManager::_('Nelze_vkladat_absence_po_ukonceni_pp'), StringUtils::format('date', $absenceRec->pp ->datumUkonceni)) );
                }
            }

            $absenceRec->absenceScript(RonPortalScriptEvents::VLOZENI_ABSENCE_BEFORE);

            $absenceRec ->provedVlozeni();
        }

        return new Redirect(null, '?menuid=monitorovani');
    }

    /**
     * @param $nastaveni
     * @return Redirect
     * @throws Exception
     */
    private function handleVlozeniDochazky($nastaveni)
    {
        $osoba = $_POST['osoba'];

        if (isset($nastaveni['typprava'])) {
            $prava = Prava::vratPrava($nastaveni['typprava']);
        }
        else {
            $prava = Prava::vratPrava('vkladani');
        }

        Prava::nastavObdobiProPrava(DateUtils::today());

        if (!$prava ->lzeZobrazitPP($osoba)) {
            RonPortalUtils::neniPravoException();
        }

        return new Redirect(null, "?menuid=vlozeni&osoba=$osoba&backaction=monitorovani");
    }

    /**
     * @param $nastaveni
     * @return Redirect
     * @throws Exception
     */
    public function handleButtonMonitorovani($nastaveni)
    {
        foreach ($nastaveni['sloupecmonitorovani'] as $sloupec) {
            if (($sloupec['typ'] === $_POST['action']) && ($sloupec['buttonid'] === $_POST['buttonid'])) {
                if ($sloupec['typ'] === 'vlozeniabsence') {
                    return $this->handleVlozeniAbsence($sloupec);
                }

                if ($sloupec['typ'] === 'vlozenidochazky') {
                    return $this->handleVlozeniDochazky($sloupec);
                }
            }
        }

        throw new Exception(StringUtils::fromUTF8('Neznama akce'));
    }

    /**
     * @param string $action
     * @return ViewObject|OutputObject|Redirect|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (isset($_GET['fotopp'])) {
            return $this ->downloadFoto( (int)$_GET['fotopp'] );
        }

        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');
        HtmlHeaders::getInstance() ->setTitle( TextManager::_('RON_Portal') . ' - ' . TextManager::_('Monitorovani'));

        if (!(RonPortalSettings::getValue(MonitorovaniOsobBrowser::SETTINGS_SLOUPCE) || RonPortalSettings::getValue('RonPortal_monitorovani_nastaveni_xml'))) {
            throw new Exception( TextManager::_('Neni_nastaveni_sloupcu_tabulky') );
        }

        if (isset($_POST['monitorovatco'])) {
            HttpResponse::setCookie('ads_monitorovatco', $_POST['monitorovatco'], time() + 30000000);
        }
        else if (isset($_GET['monitorovatco'])) {
            HttpResponse::setCookie('ads_monitorovatco', $_GET['monitorovatco'], time() + 30000000);
        }
        else if (isset($_COOKIE['ads_monitorovatco'])) {
            $_SESSION['comonitorovat'] = $_COOKIE['ads_monitorovatco'];
        }

        if (User::userAuthenticated()) {
            if (!Prava::jsouNejakePrava('monitorovani')) {
                throw new NeniPravoException();
            }
        }

        $b = new MonitorovaniOsobBrowser($this ->database, $action);

        if (isset($_POST['action'], $_POST['buttonid']) && User::userAuthenticated()) {
            return $this->handleButtonMonitorovani($b->nastaveniSloupcuXML);
        }

        $b ->load();

        if (User::userAuthenticated()) {
            // uzivatel je prihlasen => vraci ViewObject
            return $b;
        }

        Model::addViewObject($b);
        return 'view/appview_nouser.php';
    }

}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2302_169.zip: ronportal/ronportal/dochazka/monitorovanicontroller.php <==
<?php
/**
 * stanislav.elbl, 27.02.2021
 */

namespace RonPortal\Dochazka;


use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\Http\HtmlHeaders;
use Ron\Http\HttpResponse;
use Ron\MVC\HttpController;
use Ron\MVC\Model;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalScriptEvents;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\System\FotoOutputObject;
use RonPortal\User;

class MonitorovaniController extends HttpController
{
    const SETTINGS_POVOLIT_BEZ_PRIHLASENI = 'monitorovanibezprihlaseni';

    /** @var Database */
    protected $database;

    /**
     * @param Database $database
     */
    public function __construct($database)
    {
        $this ->database = $database;
        $this ->enableAccessWithoutLogin = RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_BEZ_PRIHLASENI);
    }


    /**
     * @param $idPP
     * @return FotoOutputObject
     * @throws Exception
     */
    protected function downloadFoto($idPP)
    {
        // pozor! kontrola prava se neprovadi, pokud je zapnuto monitorovani bez prihlaseni!
        if ( !RonPortalSettings::getIntValue(self::SETTINGS_POVOLIT_BEZ_PRIHLASENI) ) {
            Prava::nastavObdobiProPrava(time(), time());

            if (Prava::existujePravo('monitorovani_fotoimg')) {
                $prava = Prava::vratPrava('monitorovani_fotoimg');
            } else if (Prava::existujePravo('monitorovani_foto')) {
                $prava = Prava::vratPrava('monitorovani_foto');
            } else {
                $prava = Prava::vratPrava('monitorovani');
            }

            if ( !$prava->lzeZobrazitPP($idPP, time(), time()) ){
                throw new NeniPravoException;
            }
        }

        $result = new FotoOutputObject($this ->database);
        $result ->nactiIdFotkyPracovnihoPomeru($idPP);

        return $result;
    }

    /**
     * @param $nastaveniOperace
     * @return Redirect
     * @throws Exception
     */
    private function handleVlozeniAbsence($nastaveniOperace)
    {
        if (!is_null($nastaveniOperace)) {
            $osoba = (int)$_POST['osoba'];
            $kodOperace = $nastaveniOperace['operace'];
            $delka = $nastaveniOperace['delka'];

            if ($nastaveniOperace['delka'] > 2) {
                return new Redirect(null,
                    "?menuid=planovani&osoba=$osoba&datum=" . StringUtils::format('date', DateUtils::today()) .
                    "&delka=$delka&backaction=monitorovani&operace=" . urlencode($kodOperace));
            }

            $pPomer = PracovniPomer::nactiPP($this ->database, $osoba);

            $operaceDochazky = OperaceDochazky::nactiOperaci($this->database, $kodOperace);

            $absenceRec = new AbsenceDBRecord($this->database, $pPomer, 'monitorovani');
            $absenceRec->setLogDescription($this);

            $absenceRec->fOperace->setNewValue($operaceDochazky->kod);
            $absenceRec->popisOperace = $operaceDochazky->popis;

            $absenceRec->fDatumOd->setNewValue(DateUtils::today());
            $absenceRec->fDatumDo->setNewValue(DateUtils::today());

            $absenceRec->fDelka->setNewValue($delka);

            if ($nastaveniOperace['typprava']) {
                $prava = Prava::vratPrava($nastaveniOperace['typprava']);
            }
            else {
                $prava = Prava::vratPrava('absence', 'absence_' . $operaceDochazky->kod);
            }

            Prava::nastavObdobiProPrava(DateUtils::today());

            if (!$prava ->lzeZobrazitPP($pPomer ->idPP)) {
                throw new NeniPravoException();
            }

            $absenceRec ->fHistorie ->setNewValue(
                date('j.n.Y G:i') . ', ' . RonPortalUtils::getLogUserName(false) . ': ' .
                TextManager::_('Log_naplanovani_absence') . ' ' .
                date('j.n.Y', $absenceRec ->fDatumOd ->value) . ' - ' . date('j.n.Y', $absenceRec->fDatumDo->value) . "\n");

            if ($operaceDochazky ->autoSchvalovat) {
                $absenceRec->fSchvaleni->setNewValue(1);
            } else {
                $prava = Prava::vratPrava('absence_schval', 'absence_schval_' . $absenceRec ->fOperace ->value);
                if (RonPortalSettings::getIntValue(RonPortalSettings::ABSENCE_SCHVALOVANI)  &&  $absenceRec->pp ->lzeZobrazitPP($prava)) {
                    // muze i schvalovat
                    // nastavi schvaleni (f_updatelog = true -> provede pouze v pripade zmeny neceho jineho)
                    $absenceRec ->fSchvaleni ->setNewValue($_POST['Schvaleni'] ? 1 : 0);
                    $absenceRec ->fSchvalil ->setNewValue($_POST['Schvaleni'] ? RonPortalUtils::getLogUserName() : null);
                    $absenceRec ->fDatumSchvaleni ->setNewValue($_POST['Schvaleni'] ? time() : null);
                } else {
                    $absenceRec ->fSchvaleni ->setNewValue(0);
                }
            }

            $absenceRec ->zkontrolujDataPredZapisem(); // zkontroluje povolene delky, obdobi, atd.
            $absenceRec ->zkontrolujPrekryvani(false); // zkontroluje prekryvani absenci

//            AbsenceDBRecord::zkontrolujPovoleneDelkyOperace($this->database, $operaceDochazky->kod, $delka);
//
//            AbsenceDBRecord::zkontrolujObdobiAbsence($this ->database, $absenceRec->fDatumOd->value, $absenceRec->fDatumDo->value, $absenceRec->pp);
//
//            AbsenceDBRecord::zkontrolujPrekryvaniAbsenci($this ->database, $absenceRec->pp, $absenceRec->fOperace->value,
//                $absenceRec->fDatumOd->value, $absenceRec->fDatumDo->value, $absenceRec->fDelka->value, $absenceRec->fCasOd->value, $absenceRec->fCasDo->value);
//
//            if (RonPortalSettings::getIntValue('zakazatabsencepoukonceni')) {
//                if ($absenceRec->pp->datumUkonceni  &&  $absenceRec->fDatumDo->value > $absenceRec->pp->datumUkonceni) {
//                    throw new Exception( sprintf(TextManager::_('Nelze_vkladat_absence_po_ukonceni_pp'), StringUtils::format('date', $absenceRec->pp ->datumUkonceni)) );
//                }
//            }

            $absenceRec->absenceScript(RonPortalScriptEvents::VLOZENI_ABSENCE_BEFORE);

            $absenceRec ->provedVlozeni();
        }

        return new Redirect(null, '?menuid=monitorovani');
    }

    /**
     * @param $nastaveni
     * @return Redirect
     * @throws Exception
     */
    private function handleVlozeniDochazky($nastaveni)
    {
        $osoba = $_POST['osoba'];

        if (isset($nastaveni['typprava'])) {
            $prava = Prava::vratPrava($nastaveni['typprava']);
        }
        else {
            $prava = Prava::vratPrava('vkladani');
        }

        Prava::nastavObdobiProPrava(DateUtils::today());

        if (!$prava ->lzeZobrazitPP($osoba)) {
            RonPortalUtils::neniPravoException();
        }

        return new Redirect(null, "?menuid=vlozeni&osoba=$osoba&backaction=monitorovani");
    }

    /**
     * @param $nastaveni
     * @return Redirect
     * @throws Exception
     */
    public function handleButtonMonitorovani($nastaveni)
    {
        foreach ($nastaveni['sloupecmonitorovani'] as $sloupec) {
            if (($sloupec['typ'] === $_POST['action']) && ($sloupec['buttonid'] === $_POST['buttonid'])) {
                if ($sloupec['typ'] === 'vlozeniabsence') {
                    return $this->handleVlozeniAbsence($sloupec);
                }

                if ($sloupec['typ'] === 'vlozenidochazky') {
                    return $this->handleVlozeniDochazky($sloupec);
                }
            }
        }

        throw new Exception(StringUtils::fromUTF8('Neznama akce'));
    }

    /**
     * @param string $action
     * @return ViewObject|OutputObject|Redirect|string
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (isset($_GET['fotopp'])) {
            return $this ->downloadFoto( (int)$_GET['fotopp'] );
        }

        TextManager::getInstance() ->loadStrings('dochazka.ini');
        HtmlHeaders::getInstance() ->addScript('script/dochazka.js');
        HtmlHeaders::getInstance() ->setTitle( TextManager::_('RON_Portal') . ' - ' . TextManager::_('Monitorovani'));

        if (!(RonPortalSettings::getValue(MonitorovaniOsobBrowser::SETTINGS_SLOUPCE) || RonPortalSettings::getValue('RonPortal_monitorovani_nastaveni_xml'))) {
            throw new Exception( TextManager::_('Neni_nastaveni_sloupcu_tabulky') );
        }

        if (isset($_POST['monitorovatco'])) {
            HttpResponse::setCookie('ads_monitorovatco', $_POST['monitorovatco'], time() + 30000000);
        }
        else if (isset($_GET['monitorovatco'])) {
            HttpResponse::setCookie('ads_monitorovatco', $_GET['monitorovatco'], time() + 30000000);
        }
        else if (isset($_COOKIE['ads_monitorovatco'])) {
            $_SESSION['comonitorovat'] = $_COOKIE['ads_monitorovatco'];
        }

        if (User::userAuthenticated()) {
            if (!Prava::jsouNejakePrava('monitorovani')) {
                throw new NeniPravoException();
            }
        }

        $b = new MonitorovaniOsobBrowser($this ->database, $action);

        if (isset($_POST['action'], $_POST['buttonid']) && User::userAuthenticated()) {
            return $this->handleButtonMonitorovani($b->nastaveniSloupcuXML);
        }

        $b ->load();

        if (User::userAuthenticated()) {
            // uzivatel je prihlasen => vraci ViewObject
            return $b;
        }

        Model::addViewObject($b);
        return 'view/appview_nouser.php';
    }

}
